import 'dart:convert';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_counts_by_day_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/counts_by_day.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/counts_by_day.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardCountsByDay extends StatelessWidget {
  const CardCountsByDay({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getCountsByDayDaysCount();

    return BlocBuilder<DataCountsByDayCubit, DataCountsByDayState>(
      builder: (BuildContext context, DataCountsByDayState state) {
        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'counts_by_day'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: updateCountsByDay(daysCount),
          content: ChartCountsByDay(
            chartData: CountsByDayData.fromJson(jsonDecode(state.countsByDay.toString())),
            isLoading: false,
          ),
        );
      },
    );
  }

  Widget updateCountsByDay(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<CountsByDayData> futureCountsByDay = ScrobblesApi.fetchCountsByDay(daysCount);

    return BlocBuilder<DataCountsByDayCubit, DataCountsByDayState>(
      builder: (BuildContext context, DataCountsByDayState state) {
        return FutureBuilder<CountsByDayData>(
          future: futureCountsByDay,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataCountsByDayCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
